__title__ = 'compas_rrc'
__description__ = 'COMPAS RRC: Library for ABB Robots'
__url__ = 'https://github.com/compas-rrc/compas_rrc'
__version__ = '1.1.0'
__author__ = 'ETH Zurich'
__author_email__ = 'rrc@arch.ethz.ch'
__license__ = 'MIT license'
__copyright__ = 'Copyright 2019 ETH Zurich'

__all__ = ['__author__', '__author_email__', '__copyright__', '__description__', '__license__', '__title__', '__url__', '__version__']
